package com.example.zcatalyst_sdk

import android.content.Context
import android.graphics.Color
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import com.zoho.accounts.clientframework.IAMClientSDK
import com.zoho.accounts.clientframework.IAMErrorCodes
import com.zoho.accounts.clientframework.IAMToken
import com.zoho.accounts.clientframework.IAMTokenCallback
import io.flutter.Log
import io.flutter.Log.*
import java.io.IOException
import java.util.*
import kotlin.Exception
import kotlin.IllegalArgumentException
import kotlin.collections.HashMap

class MethodCallHandler(private var applicationContext: Context) :
    MethodChannel.MethodCallHandler {

    private val iamSDK: IAMClientSDK = IAMClientSDK.getInstance(applicationContext)

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            IAMSDKConstants.MethodCall.INIT -> {
                val environment: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.ENVIRONMENT)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.ENVIRONMENT,
                            null
                        )
                val isCustomLogin: Boolean =
                    call.argument<Boolean>(IAMSDKConstants.MethodArguments.IS_CUSTOM_LOGIN)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.IS_CUSTOM_LOGIN,
                            null
                        )
                init(isCustomLogin, ZCatalystEnvironment.of(environment), result)
            }
            IAMSDKConstants.MethodCall.INIT_WITH_CONFIGS -> {
                val accountsPortalBaseURL: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.ACCOUNTS_PORTAL_URL)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.ACCOUNTS_PORTAL_URL,
                            null
                        )
                val clientID: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.CLIENT_ID)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.CLIENT_ID,
                            null
                        )
                val clientSecret: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.CLIENT_SECRET)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.CLIENT_SECRET,
                            null
                        )
                val portalID: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.PORTAL_ID)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.PORTAL_ID,
                            null
                        )
                val redirectURLScheme: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.REDIRECT_URL_SCHEME)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.REDIRECT_URL_SCHEME,
                            null
                        )
                val scopes: String = call.argument<String>(IAMSDKConstants.MethodArguments.SCOPES)
                    ?: return result.error(
                        IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                        IAMSDKConstants.ErrorMessage.SCOPES,
                        null
                    )
                init(
                    accountsPortalBaseURL,
                    redirectURLScheme,
                    clientID,
                    clientSecret,
                    portalID,
                    scopes,
                    result
                )
            }
            IAMSDKConstants.MethodCall.LOGIN -> {
                val customParams: HashMap<String, String>? =
                    call.argument(IAMSDKConstants.MethodArguments.CUSTOM_PARAMS)
                login(customParams, result)
            }
            IAMSDKConstants.MethodCall.IS_USER_LOGGED_IN -> {
                isUserSignedIn(result)
            }
            IAMSDKConstants.MethodCall.GET_ACCESS_TOKEN -> {
                getAccessToken(result)
            }
            IAMSDKConstants.MethodCall.GET_OAUTH_TOKEN_WITH_JWT_TOKEN -> {
                val jwtToken: String =
                    call.argument<String>(IAMSDKConstants.MethodArguments.JWT_TOKEN)
                        ?: return result.error(
                            IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                            IAMSDKConstants.ErrorMessage.JWT_TOKEN,
                            null
                        )
                getAccessTokenWithJWTToken(jwtToken, result)
            }
            IAMSDKConstants.MethodCall.LOGOUT -> {
                logout(result)
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    private fun init(
        isCustomLogin: Boolean,
        environment: ZCatalystEnvironment,
        result: MethodChannel.Result
    ) {

        try {
            val properties: Properties = getProperties(environment)
            val projectID: Long = properties.getProperty("projectID").toLong()
            val accountsPortalBaseURL: String = properties.getProperty("AccountsPortalDomain")
            val scopes: String = properties.getProperty("oauthScopes")
            val clientID: String = properties.getProperty("clientID")
            val clientSecret: String = properties.getProperty("clientSecret")
            val jwtClientID: String? =
                if (properties.getProperty("JWTClientID") == IAMSDKConstants.CONFIG_PROPERTY_NOT_AVAILABLE) {
                    null
                } else {
                    properties.getProperty("JWTClientID")
                }
            val jwtClientSecret: String? =
                if (properties.getProperty("JWTClientSecret") == IAMSDKConstants.CONFIG_PROPERTY_NOT_AVAILABLE) {
                    null
                } else {
                    properties.getProperty("JWTClientSecret")
                }
            val redirectUrlScheme: String = properties.getProperty("redirectUrl") + "://"
            val portalID: String = properties.getProperty("portalID")
            val apiBaseUrl: String = properties.getProperty("apiBaseURL")
            val requestHeaders: HashMap<String, String> =
                getValuesAsMap(properties.getProperty("requestHeaders"))
            requestHeaders.put(
                "PROJECT_ID",
                portalID
            )
            val printStackTrace: Boolean = properties.getProperty("printStackTrace").toBoolean()

            if (!isCustomLogin) {
                iamSDK.init(
                    accountsPortalBaseURL,
                    redirectUrlScheme,
                    clientID,
                    clientSecret,
                    portalID,
                    scopes
                )
            } else if (jwtClientID != null && jwtClientSecret != null) {
                iamSDK.init(
                    accountsPortalBaseURL,
                    redirectUrlScheme,
                    jwtClientID,
                    jwtClientSecret,
                    portalID,
                    scopes
                )
            } else {
                return result.error(
                    IAMSDKConstants.ErrorCodes.CUSTOM_LOGIN_DISABLED,
                    IAMSDKConstants.ErrorMessage.CUSTOM_LOGIN_DISABLED,
                    null
                )
            }
            val configsMap: HashMap<String, Any?> = hashMapOf(
                IAMSDKConstants.SDKConfigKeys.ACCOUNTS_PORTAL_DOMAIN to accountsPortalBaseURL,
                IAMSDKConstants.SDKConfigKeys.PROJECT_ID to projectID,
                IAMSDKConstants.SDKConfigKeys.CLIENT_ID to clientID,
                IAMSDKConstants.SDKConfigKeys.CLIENT_SECRET to clientSecret,
                IAMSDKConstants.SDKConfigKeys.JWT_CLIENT_ID to jwtClientID,
                IAMSDKConstants.SDKConfigKeys.JWT_CLIENT_SECRET to jwtClientSecret,
                IAMSDKConstants.SDKConfigKeys.REDIRECT_URL_SCHEME to redirectUrlScheme,
                IAMSDKConstants.SDKConfigKeys.SCOPES to scopes,
                IAMSDKConstants.SDKConfigKeys.PORTAL_ID to portalID,
                IAMSDKConstants.SDKConfigKeys.API_BASE_URL to apiBaseUrl,
                IAMSDKConstants.SDKConfigKeys.REQUEST_HEADERS to requestHeaders,
                IAMSDKConstants.SDKConfigKeys.PRINT_STACK_TRACE to printStackTrace,
            )
            result.success(configsMap)
        } catch (exception: IOException) {
            return result.error(
                IAMSDKConstants.ErrorCodes.INITIALIZATION_ERROR,
                exception.toString(),
                null
            )
        } catch (exception: IllegalArgumentException) {
            return result.error(IAMSDKConstants.ErrorCodes.INTERNAL_ERROR, exception.message, null)
        } catch (exception: NumberFormatException) {
            return result.error(
                IAMSDKConstants.ErrorCodes.INTERNAL_ERROR,
                exception.toString(),
                null
            )
        }
    }

    private fun init(
        accountsPortalBaseURL: String,
        redirectURLScheme: String,
        clientID: String,
        clientSecret: String,
        portalID: String,
        scopes: String,
        result: MethodChannel.Result
    ) {

        iamSDK.init(
            accountsPortalBaseURL,
            redirectURLScheme,
            clientID,
            clientSecret,
            portalID,
            scopes
        )
        result.success(null)
    }

    private fun getProperties(environment: ZCatalystEnvironment): Properties {

        val fileName: String =
            if (environment.equals(ZCatalystEnvironment.PRODUCTION)) {
                "app_configuration_production.properties"
            } else {
                "app_configuration_development.properties"
            }
        val inputStream = this.applicationContext.assets.open(fileName)
        val properties = Properties()
        inputStream.apply { properties.load(inputStream) }

        val mandatoryConfigs =
            listOf(
                "clientID",
                "clientSecret",
                "redirectUrl",
                "projectID",
                "portalID",
                "AccountsPortalDomain"
            )
        if (!properties.stringPropertyNames().containsAll(mandatoryConfigs)) {
            throw IllegalArgumentException(
                IAMSDKConstants.ErrorMessage.MANDATORY_CONFIG_MISSING + " - (" + mandatoryConfigs.toString() + ")."
            )
        }
        return properties
    }

    private fun getValuesAsMap(values: String?): HashMap<String, String> {
        val valuesAsMap = HashMap<String, String>()
        val valuesList = values?.split(",")
        valuesList?.forEach { valueString ->
            val value = valueString.split(":")
            if (value.size == 2) {
                valuesAsMap[value[0]] = value[1]
            }
        }
        return valuesAsMap
    }

    private fun login(params: HashMap<String, String>?, result: MethodChannel.Result) {
        when {
            iamSDK.isUserSignedIn -> {
                Log.i(IAMSDKConstants.LOG_TAG, "User already signed in..")
                iamSDK.getToken(getIAMTokenCallback(result))
            }
            else -> {
                Log.i(IAMSDKConstants.LOG_TAG, "Presenting Login Screen..")
                iamSDK.presentLoginScreen(
                    this.applicationContext,
                    getIAMTokenCallback(result),
                    Color.parseColor("#757575"),
                    params
                )
            }
        }
    }

    private fun isUserSignedIn(result: MethodChannel.Result) {
        result.success(iamSDK.isUserSignedIn)
    }

    private fun getAccessToken(result: MethodChannel.Result) {
        iamSDK.getToken(object : IAMTokenCallback {

            override fun onTokenFetchInitiated() {
                Log.i(IAMSDKConstants.LOG_TAG, "Token fetch initiated...")
            }

            override fun onTokenFetchComplete(iamToken: IAMToken) {
                Log.i(IAMSDKConstants.LOG_TAG, "Token fetch Success...")
                result.success(iamToken.token)
            }

            override fun onTokenFetchFailed(iamErrorCodes: IAMErrorCodes) {
                Log.e(
                    IAMSDKConstants.LOG_TAG,
                    IAMSDKConstants.ErrorCodes.TOKEN_FETCH_ERROR + ": " + iamErrorCodes.description
                )
                result.error(
                    IAMSDKConstants.ErrorCodes.TOKEN_FETCH_ERROR + ": " + iamErrorCodes.name,
                    iamErrorCodes.description,
                    null
                )
            }
        })
    }

    private fun getAccessTokenWithJWTToken(jwtToken: String, result: MethodChannel.Result) {
        iamSDK.getRemoteTokenWithJWTToken(
            jwtToken, object : IAMTokenCallback {

                override fun onTokenFetchInitiated() {
                    Log.i(IAMSDKConstants.LOG_TAG, "Token fetch initiated...")
                }

                override fun onTokenFetchComplete(iamToken: IAMToken) {
                    Log.i(IAMSDKConstants.LOG_TAG, "Token fetch Success...")
                    result.success(null)
                }

                override fun onTokenFetchFailed(iamErrorCodes: IAMErrorCodes) {
                    Log.e(
                        IAMSDKConstants.LOG_TAG,
                        IAMSDKConstants.ErrorCodes.TOKEN_FETCH_ERROR + ": " + iamErrorCodes.description
                    )
                    result.error(
                        IAMSDKConstants.ErrorCodes.TOKEN_FETCH_ERROR + ": " + iamErrorCodes.name,
                        iamErrorCodes.description,
                        null
                    )
                }
            })
    }

    private fun logout(result: MethodChannel.Result) {

        iamSDK.revoke(object : IAMClientSDK.OnLogoutListener {
            override fun onLogoutSuccess() {
                result.success(null)
            }

            override fun onLogoutFailed() {
                Log.e(
                    IAMSDKConstants.LOG_TAG,
                    IAMSDKConstants.ErrorCodes.LOGOUT_FAILED + ": " + IAMSDKConstants.ErrorMessage.LOGOUT_FAILED
                )
                result.error(
                    IAMSDKConstants.ErrorCodes.LOGOUT_FAILED,
                    IAMSDKConstants.ErrorMessage.LOGOUT_FAILED,
                    null
                )
            }
        })
    }

    private fun getIAMTokenCallback(
        result: MethodChannel.Result
    ): IAMTokenCallback {
        return object : IAMTokenCallback {

            override fun onTokenFetchInitiated() {
                Log.i(IAMSDKConstants.LOG_TAG, "Token fetch initiated...")
            }

            override fun onTokenFetchComplete(iamToken: IAMToken) {
                Log.i(IAMSDKConstants.LOG_TAG, "Token fetch Success...")
                result.success(null)
            }

            override fun onTokenFetchFailed(iamErrorCodes: IAMErrorCodes) {
                Log.e(
                    IAMSDKConstants.LOG_TAG,
                    IAMSDKConstants.ErrorCodes.LOGIN_ERROR + ": " + iamErrorCodes.description
                )
                result.error(
                    IAMSDKConstants.ErrorCodes.LOGIN_ERROR + ": " + iamErrorCodes.name,
                    iamErrorCodes.description,
                    null
                )
            }
        }
    }

}


