import Flutter
import UIKit

public class ZCatalystSDKPlugin: NSObject, FlutterPlugin{

    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "\(IAMSDKConstants.MethodChannel.domain)/\(IAMSDKConstants.MethodChannel.name)", binaryMessenger: registrar.messenger())
        let instance = ZCatalystSDKPlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
        registrar.addApplicationDelegate(instance)

    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {

        do{
            switch(call.method){

            case IAMSDKConstants.MethodCall.initWithPlistFile:
                guard let dartArguments = call.arguments as? Dictionary<String, Any> else {
                    return result(FlutterError(code: IAMSDKConstants.ErrorCodes.internalError, message: "\(IAMSDKConstants.ErrorMessage.missingDartArguments) \(IAMSDKConstants.MethodCall.initWithPlistFile)", details: nil))
                }
                let environment: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.environment)
                let isCustomLogin: Bool = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.isCustomLogin)
                self.initialize(isCustomLogin, ZCatalystEnvironment.of(environment), result)
            case IAMSDKConstants.MethodCall.initWithConfigs:
                guard let dartArguments = call.arguments as? Dictionary<String, Any> else {
                    return result(FlutterError(code: IAMSDKConstants.ErrorCodes.internalError, message: "\(IAMSDKConstants.ErrorMessage.missingDartArguments) \(IAMSDKConstants.MethodCall.initWithConfigs)", details: nil))
                }
                let accountsURL: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.accountsPortalUrl)
                let redirectURLScheme: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.redirectUrlScheme)
                let clientID: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.clientId)
                let clientSecret: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.clientSecret)
                let portalID: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.portalId)
                let scopes: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.scopes)
                self.initialize(accountsURL, clientID, clientSecret, redirectURLScheme, portalID, scopes, result)
            case IAMSDKConstants.MethodCall.login:
                guard let dartArguments = call.arguments as? Dictionary<String, Any> else {
                    return result(FlutterError(code: IAMSDKConstants.ErrorCodes.internalError, message: "\(IAMSDKConstants.ErrorMessage.missingDartArguments) \(IAMSDKConstants.MethodCall.login)", details: nil))
                }
                let customParams: Dictionary<String, Any>? = dartArguments[IAMSDKConstants.MethodArguments.customParams] as? Dictionary<String, Any>
                self.login(customParams, result)
            case IAMSDKConstants.MethodCall.isUserLoggedIn:
                result(self.isUserLoggedIn())
            case IAMSDKConstants.MethodCall.getOauthTokenWithJwtToken:
                guard let dartArguments = call.arguments as? Dictionary<String, Any> else {
                    return result(FlutterError(code: IAMSDKConstants.ErrorCodes.internalError, message: "\(IAMSDKConstants.ErrorMessage.missingDartArguments) \(IAMSDKConstants.MethodCall.login)", details: nil))
                }
                let jwtToken: String = try dartArguments.getValue(key: IAMSDKConstants.MethodArguments.jwtToken)
                self.getOauthToken(withJWTToken: jwtToken, result)
            case IAMSDKConstants.MethodCall.getAccessToken:
                self.getAccessToken(result)
            case IAMSDKConstants.MethodCall.logout:
                self.logout(result)
            default:
                result(FlutterMethodNotImplemented)
            }
        }
        catch {
            if let error = error as? ZCatalystError, case ZCatalystError.processingError(let code, let message, let details) = error
            {
                result(FlutterError(code: code, message: message, details: details))
            }
        }
    }

    public func application(_ application: UIApplication, open url: URL, options: [UIApplication.OpenURLOptionsKey : Any] = [:]) -> Bool {
        let handled: Bool = ZohoPortalAuth.handleURL(url, sourceApplication: options[.sourceApplication], annotation: options[.annotation])
        return handled
    }

    func initialize(_ isCustomLogin: Bool, _ environment: ZCatalystEnvironment, _ result: FlutterResult) {
        let configSettings: ZCatalystSDKConfigs
        do{
            configSettings = try getProperties(environment, result)
        }catch{
            return result(FlutterError(code: IAMSDKConstants.ErrorCodes.fileNotFound, message: IAMSDKConstants.ErrorMessage.fileNotFound, details: nil))
        }
        if(!isCustomLogin){
            ZohoPortalAuth.initWithClientID(configSettings.clientID, clientSecret: configSettings.clientSecretID, portalID: configSettings.portalID, scope: configSettings.OAuthScopes, urlScheme: configSettings.redirectURLScheme, mainWindow: UIApplication.shared.windows.first , accountsPortalURL: configSettings.accountsURL)
        } else if(configSettings.jwtClientID != IAMSDKConstants.configPropertyNotAvailable && configSettings.jwtClientSecret != IAMSDKConstants.configPropertyNotAvailable){
            ZohoPortalAuth.initWithClientID(configSettings.jwtClientID, clientSecret: configSettings.jwtClientSecret, portalID: configSettings.portalID, scope: configSettings.OAuthScopes, urlScheme: configSettings.redirectURLScheme, mainWindow: UIApplication.shared.windows.first, accountsPortalURL: configSettings.accountsURL)
        }
        else{
            return result(FlutterError(code: IAMSDKConstants.ErrorCodes.customLoginDisabled, message: IAMSDKConstants.ErrorCodes.customLoginDisabled, details: nil))
        }
        let configsMap: Dictionary<String, Any> = [
            IAMSDKConstants.SDKConfigKeys.accountsPortalDomain: configSettings.accountsURL,
            IAMSDKConstants.SDKConfigKeys.projectId: Int(configSettings.projectID) ?? 0,
            IAMSDKConstants.SDKConfigKeys.clientId: configSettings.clientID,
            IAMSDKConstants.SDKConfigKeys.clientSecret: configSettings.clientSecretID,
            IAMSDKConstants.SDKConfigKeys.jwtClientId: configSettings.jwtClientID,
            IAMSDKConstants.SDKConfigKeys.jwtClientSecret: configSettings.jwtClientSecret,
            IAMSDKConstants.SDKConfigKeys.portalId: configSettings.portalID,
            IAMSDKConstants.SDKConfigKeys.redirectUrlScheme: configSettings.redirectURLScheme,
            IAMSDKConstants.SDKConfigKeys.scopes: configSettings.oAuthScopeString,
            IAMSDKConstants.SDKConfigKeys.apiBaseUrl: configSettings.apiBaseURL,
            IAMSDKConstants.SDKConfigKeys.requestHeaders: configSettings.requestHeaders
        ]
        result(configsMap)

    }

    func initialize(_ accountsURL: String, _ clientID: String, _ clientSecret: String, _ redirectURLScheme: String, _ portalID: String, _ scopes: String, _ result: FlutterResult){
        ZohoPortalAuth.initWithClientID(clientID, clientSecret: clientSecret, portalID: portalID, scope: scopes.components(separatedBy: ","), urlScheme: redirectURLScheme, mainWindow: UIApplication.shared.windows.first, accountsPortalURL: accountsURL)
        result(nil)

    }

    func getProperties(_ environment: ZCatalystEnvironment, _ result: FlutterResult) throws -> ZCatalystSDKConfigs{
        let fileName: String
        if(environment == .DEVELOPMENT){
            fileName = "AppConfigurationDevelopment"
        }
        else{
            fileName = "AppConfigurationProduction"
        }
        let path = Bundle.main.path(forResource: fileName, ofType: "plist")
        guard let aPath = path else {
            throw ZCatalystError.fileNotFound
        }
        let decoder = PropertyListDecoder()
        guard let fileData =  FileManager.default.contents(atPath: aPath) else {
            throw ZCatalystError.fileNotFound
        }
        guard let configSettings = try? decoder.decode(ZCatalystSDKConfigs.self, from: fileData) else {
            throw ZCatalystError.fileNotFound
        }
        return configSettings
    }

    func login(_ customParams: Dictionary<String, Any>?, _ result: @escaping FlutterResult){
        ZohoPortalAuth.presentZohoPortalSign { (token, error) in
            if token != nil && error == nil
            {
                return result(nil)
            }
            result(FlutterError(code: IAMSDKConstants.ErrorCodes.loginFailed, message: error.debugDescription, details: nil))
        }
    }

    func getOauthToken(withJWTToken jwtToken: String, _ result: @escaping FlutterResult){
        ZohoPortalAuth.getOAuthToken(forRemoteUser: jwtToken, tokenType: ZohoPortalAuthRemoteLoginTypeJWT, baseURL: nil, tokenHandler: { (oauthToken, err) in
            if let error = err {
                return result(FlutterError(code: IAMSDKConstants.ErrorCodes.tokenFetchError, message: error.localizedDescription, details: nil))
            }
            result(nil)
        }
        )}

    func isUserLoggedIn() -> Bool{
        return ZohoPortalAuth.isUserSignedIn()
    }

    func getAccessToken(_ result: @escaping FlutterResult){
        ZohoPortalAuth.getOauth2Token { (token, error) in
            if token != nil && error == nil
            {
                return result(token)
            }
            result(FlutterError(code: IAMSDKConstants.ErrorCodes.tokenFetchError, message: error.debugDescription, details: nil))
        }
    }

    func logout(_ result: @escaping FlutterResult){
        ZohoPortalAuth.revokeAccessToken { (error) in
            if error == nil
            {
                return result(nil)
            }
            result(FlutterError(code: IAMSDKConstants.ErrorCodes.logoutFailed, message: error.debugDescription, details: nil))
        }
    }

}
