import 'dart:async';
import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:http/http.dart' as http;

import '../exception/zcatalyst_exception.dart';
import '../exception/zcatalyst_logger.dart';
import '../exception/zcatalyst_sdk_exception.dart';
import '../setup/zcatalyst_app.dart';
import '../util/api_constants.dart';
import 'response/base_api_response.dart';
import './../crud/zcatalyst_column.dart';
import './../filestore/zcatalyst_file.dart';
import './../crud/zcatalyst_user_delegate.dart';
import './../filestore/zcatalyst_folder.dart';
import '../zcql/zcatalyst_select_query.dart';
import './../crud/zcatalyst_role.dart';
import './../crud/zcatalyst_user.dart';
import 'package:zcatalyst_sdk/src/util/pair.dart';

part './handlers/column_api_handler.dart';
part './handlers/file_api_handler.dart';
part './handlers/folder_api_handler.dart';
part './handlers/function_api_handler.dart';
part './handlers/row_api_handler.dart';
part '../datastore/zcatalyst_data_store.dart';
part '../datastore/zcatalyst_table.dart';
part '../crud/zcatalyst_row.dart';
part '../search/zcatalyst_search_options.dart';
part '../api/handlers/search_api_handler.dart';
part '../api/handlers/user_api_handler.dart';
part '../api/handlers/zcql_api_handler.dart';
part '../util/zcatalyst_util.dart';

class APIRequest {
  APIRequest._({
    required HttpMethod method,
    required String endPoint,
    Map<String, dynamic>? params,
    Object? body,
  }) {
    _method = method;
    _endPoint = endPoint;
    _params = params ?? {};
    _body = body;
    _headers = ZCatalystApp.configs.requestHeaders;
    _url += _endPoint;
    if (_params.isNotEmpty) {
      _url += '?${queryParamsAsString(_params)}';
    }
  }

  APIRequest({required String url}) {
    _url = url;
  }

  HttpMethod _method = HttpMethod.GET;

  String _url =
      '${ZCatalystApp.configs.apiBaseURL}/baas/${ZCatalystApp.configs.apiVersion}/project/${ZCatalystApp.configs.projectID}/';

  String get url => _url;

  String _endPoint = APIConstant.STRING_MOCK;

  Map<String, dynamic> _params = {};

  Map<String, String> _headers = {};

  Object? _body;

  Future<void> _sendRequest(
      {required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      Uri uri = Uri.parse(_url);
      http.Response response;
      switch (_method) {
        case HttpMethod.GET:
          response = await http.get(uri, headers: {
            APIConstant.AUTHORIZATION:
                '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
            ..._headers
          });
          break;
        case HttpMethod.POST:
          response = await http.post(uri, body: jsonEncode(body), headers: {
            APIConstant.CONTENT_TYPE: APIConstant.JSON_CONTENT_TYPE,
            APIConstant.AUTHORIZATION:
                '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
            ..._headers
          });
          break;
        case HttpMethod.PUT:
          response = await http.put(uri, body: jsonEncode(body), headers: {
            APIConstant.CONTENT_TYPE: APIConstant.JSON_CONTENT_TYPE,
            APIConstant.AUTHORIZATION:
                '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
            ..._headers
          });
          break;
        case HttpMethod.DELETE:
          response = await http.delete(uri, headers: {
            APIConstant.CONTENT_TYPE: APIConstant.JSON_CONTENT_TYPE,
            APIConstant.AUTHORIZATION:
                '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
            ..._headers
          });
          break;
      }
      APIResponse apiResponse = APIResponse(response);
      onSuccess(apiResponse);
    } on SocketException {
      ZCatalystLogger.logError(ErrorMessage.OFFLINE);
      onFailed(ZCatalystException(
          code: ErrorCode.NO_NETWORK_AVAILABLE,
          errorMessage: ErrorMessage.OFFLINE));
    } on FormatException catch (ex) {
      ZCatalystLogger.logError(ex.message);
      onFailed(ZCatalystSDKException(originalException: ex));
    } on IOException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ZCatalystException(
          code: ErrorCode.API_FAILED, originalException: ex));
    } on ZCatalystException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ex);
    } catch (e) {
      onFailed(ZCatalystSDKException(message: e.toString()));
    }
  }

  Future<void> _uploadFile(
      {required String filePath,
      required Map<String, String> body,
      required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      Uri uri = Uri.parse(_url);
      var request = http.MultipartRequest(_method.toString(), uri);
      request.headers.addAll({
        APIConstant.AUTHORIZATION:
            '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
        ..._headers
      });
      request.files.addAll([
        await http.MultipartFile.fromPath('code', filePath),
      ]);
      request.fields.addAll(body);
      http.StreamedResponse streamedResponse = await request.send();
      http.Response response = await http.Response.fromStream(streamedResponse);
      APIResponse apiResponse = APIResponse(response);
      onSuccess(apiResponse);
    } on SocketException {
      ZCatalystLogger.logError(ErrorMessage.OFFLINE);
      onFailed(ZCatalystException(
          code: ErrorCode.NO_NETWORK_AVAILABLE,
          errorMessage: ErrorMessage.OFFLINE));
    } on FormatException catch (ex) {
      ZCatalystLogger.logError(ex.message);
      onFailed(ZCatalystSDKException(originalException: ex));
    } on IOException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ZCatalystException(
          code: ErrorCode.API_FAILED, originalException: ex));
    } on ZCatalystException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ex);
    }
  }

  Future<void> _downloadFile(
      {required String filePath,
      required String fileName,
      required Function(FileAPIResponse response, String filePath) onSuccess,
      required Function(int bytesWritten, int contentLength, double percentage)
          onProgress,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      Uri uri = Uri.parse(_url);
      http.Request request = http.Request(_method.toString(), uri);
      request.headers.addAll({
        APIConstant.CONTENT_TYPE: APIConstant.JSON_CONTENT_TYPE,
        APIConstant.AUTHORIZATION:
            '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
        ..._headers
      });
      http.StreamedResponse streamedResponse = await request.send();
      FileAPIResponse fileApiResponse = FileAPIResponse(streamedResponse);
      int fileLengthInBytes = fileApiResponse.fileSize ?? -1;
      List<int> bytes = [];
      StreamSubscription<List<int>> fileDownloadSubscription =
          streamedResponse.stream.listen(
        (List<int> newBytes) {
          bytes.addAll(newBytes);
          int bytesWritten = bytes.length;
          double downloadedPercentage =
              (bytesWritten.toDouble() * 100) / fileLengthInBytes;
          onProgress(bytesWritten, fileLengthInBytes, downloadedPercentage);
        },
      );
      fileDownloadSubscription.onDone(
        () async {
          File file = File('$filePath/$fileName');
          file.writeAsBytes(bytes);
          await fileDownloadSubscription.cancel();
          onSuccess(fileApiResponse, file.path);
        },
      );
    } on SocketException {
      ZCatalystLogger.logError(ErrorMessage.OFFLINE);
      onFailed(ZCatalystException(
          code: ErrorCode.NO_NETWORK_AVAILABLE,
          errorMessage: ErrorMessage.OFFLINE));
    } on FormatException catch (ex) {
      ZCatalystLogger.logError(ex.message);
      onFailed(ZCatalystSDKException(originalException: ex));
    } on IOException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ZCatalystException(
          code: ErrorCode.API_FAILED, originalException: ex));
    } on ZCatalystException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ex);
    }
  }

  Future<void> _downloadFileAsStream(
      {required Function(FileAPIResponse response, http.ByteStream stream)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      Uri uri = Uri.parse(_url);
      http.Request request = http.Request(_method.toString(), uri);
      request.headers.addAll({
        APIConstant.AUTHORIZATION:
            '${APIConstant.AUTHORIZATION_TYPE} ${await ZCatalystApp.getInstance().getAccessToken()}',
        ..._headers
      });
      http.StreamedResponse streamedResponse = await request.send();
      FileAPIResponse fileApiResponse = FileAPIResponse(streamedResponse);
      http.ByteStream fileAsStream = streamedResponse.stream;
      onSuccess(fileApiResponse, fileAsStream);
    } on SocketException {
      ZCatalystLogger.logError(ErrorMessage.OFFLINE);
      onFailed(ZCatalystException(
          code: ErrorCode.NO_NETWORK_AVAILABLE,
          errorMessage: ErrorMessage.OFFLINE));
    } on FormatException catch (ex) {
      ZCatalystLogger.logError(ex.message);
      onFailed(ZCatalystSDKException(originalException: ex));
    } on IOException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ZCatalystException(
          code: ErrorCode.API_FAILED, originalException: ex));
    } on ZCatalystException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ex);
    }
  }

  String queryParamsAsString(Map<String, dynamic> params) {
    return Uri(queryParameters: params).query;
  }

  APIRequest setHttpMethod(HttpMethod method) {
    _method = method;
    return this;
  }

  ///The Request method / type.
  ///
  ///Possible Values: [HttpMethod.GET], [HttpMethod.POST], [HttpMethod.PUT], [HttpMethod.DELETE].
  HttpMethod get httpMethod => _method;

  APIRequest setQueryParam(String name, String value) {
    _params = <String, dynamic>{};
    _params[name] = value;
    return this;
  }

  ///The Query Parameters of an API request.
  Map<String, dynamic> get queryParams => _params;

  APIRequest setHeader(String name, String value) {
    _headers[name] = value;
    return this;
  }

  ///The headers of the current API request.
  Map<String, String> get headers => _headers;

  APIRequest setBody(Object body) {
    _body = body;
    return this;
  }

  ///The body of the current API request.
  Object? get body => _body;

  ///Sends an api request based on the given request details.
  ///On success, The **onSuccess** handler is called with an [APIResponse] object.
  ///
  ///On errors, The **OnFailed** handler is called with a [ZCatalystException].
  Future<void> getResponse(
      {required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      if (queryParams.isNotEmpty) {
        _url += '?${queryParamsAsString(queryParams)}';
      }
      Uri uri = Uri.parse(_url);
      http.Response response;
      switch (_method) {
        case HttpMethod.GET:
          response = await http.get(uri, headers: headers);
          break;
        case HttpMethod.POST:
          response = await http.post(uri, body: body, headers: headers);
          break;
        case HttpMethod.PUT:
          response = await http.put(uri, body: body, headers: headers);
          break;
        case HttpMethod.DELETE:
          response = await http.delete(uri, headers: headers);
          break;
      }
      APIResponse apiResponse = APIResponse(response);
      onSuccess(apiResponse);
    } on SocketException {
      ZCatalystLogger.logError(ErrorMessage.OFFLINE);
      onFailed(ZCatalystException(
          code: ErrorCode.NO_NETWORK_AVAILABLE,
          errorMessage: ErrorMessage.OFFLINE));
    } on FormatException catch (ex) {
      ZCatalystLogger.logError(ex.message);
      onFailed(ZCatalystSDKException(originalException: ex));
    } on IOException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ZCatalystException(
          code: ErrorCode.API_FAILED, originalException: ex));
    } on ZCatalystException catch (ex) {
      ZCatalystLogger.logError(ex.toString());
      onFailed(ex);
    }
  }

  @override
  String toString() {
    Map<String, dynamic> reqHeaders = _headers;
    reqHeaders["Authorization"] = " ******* ";
    String requestDetails =
        '${APIConstant.URL} = ${url.toString()} , ${APIConstant.HEADERS} = ${headers.toString()} , ${APIConstant.PARAMS} = ${queryParams.toString()} , ${APIConstant.REQUEST_BODY} = ${body.toString()}.';
    return requestDetails;
  }
}
