part of '../api_request.dart';

class RowAPIHandler {
  final String tableIdentifier;

  RowAPIHandler({required this.tableIdentifier});

  Future<void> getRow(
      {required int id,
      required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}/$id',
      method: HttpMethod.GET,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      ZCatalystRow row = fromJson(responseData, onFailed);
      return onSuccess(response, row);
    }, onFailed: (ZCatalystException exception) {
      return onFailed(exception);
    });
  }

  Future<void> getRows(
      {int? maxRows,
      String? nextToken,
      required Function(
              APIResponse response, List<ZCatalystRow> rows, ResponseInfo info)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}',
      method: HttpMethod.GET,
      params: {
        if (maxRows != null) APIConstant.MAX_ROWS_PARAM: '$maxRows',
        if (nextToken != null) APIConstant.NEXT_TOKEN_PARAM: nextToken
      },
    )._sendRequest(
      onSuccess: (APIResponse response) {
        var responseData = response.responseJson[APIConstant.DATA];
        List<ZCatalystRow> rows = rowFromJson(responseData, onFailed);
        ResponseInfo info = ResponseInfo(response.responseJson["more_records"],
            response.responseJson["next_token"]);
        return onSuccess(response, rows, info);
      },
      onFailed: (ZCatalystException exception) {
        return onFailed(exception);
      },
    );
  }

  Future<void> createRow(
      {required ZCatalystRow row,
      required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    var jsonArray = [];
    jsonArray.add(row._upsertMap);
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}',
      method: HttpMethod.POST,
      body: jsonArray,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      ZCatalystRow row =
          fromJson(responseData[0] as Map<String, dynamic>, onFailed);
      return onSuccess(response, row);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> updateRow(
      {required ZCatalystRow row,
      required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    var jsonArray = [];
    Map<String, dynamic> updateData = row._upsertMap;
    updateData['ROWID'] = row.id;
    jsonArray.add(updateData);

    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}',
      method: HttpMethod.PUT,
      body: jsonArray,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA][0];
      ZCatalystRow row = fromJson(responseData, onFailed);
      return onSuccess(response, row);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> deleteRow(
      {required var id,
      required Function(
        APIResponse response,
      )
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}/$id',
      method: HttpMethod.DELETE,
    )._sendRequest(onSuccess: (APIResponse response) {
      onSuccess(response);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> createRows(
      {required List<ZCatalystRow> rows,
      required Function(APIResponse response, List<ZCatalystRow> row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    var jsonArray = [];
    for (var row in rows) {
      jsonArray.add(row._upsertMap);
    }
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}',
      method: HttpMethod.POST,
      body: jsonArray,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      List<ZCatalystRow> rows = rowFromJson(responseData, onFailed);
      return onSuccess(response, rows);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  Future<void> updateRows(
      {required List<ZCatalystRow> rows,
      required Function(APIResponse response, List<ZCatalystRow> rows)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    var jsonArray = [];
    for (var row in rows) {
      Map<String, dynamic> updateData = row._upsertMap;
      updateData['ROWID'] = row.id;
      jsonArray.add(updateData);
    }
    await APIRequest._(
      endPoint: '${Endpoints.TABLE}/$tableIdentifier/${Endpoints.ROW}',
      method: HttpMethod.PUT,
      body: jsonArray,
    )._sendRequest(onSuccess: (APIResponse response) {
      var responseData = response.responseJson[APIConstant.DATA];
      List<ZCatalystRow> rows = rowFromJson(responseData, onFailed);
      return onSuccess(response, rows);
    }, onFailed: (ZCatalystException exception) {
      onFailed(exception);
    });
  }

  List<ZCatalystRow> rowFromJson(List<dynamic> data,
          Function(ZCatalystException exception) onFailed) =>
      List<ZCatalystRow>.from(
          data.map<ZCatalystRow>((x) => fromJson(x, onFailed)));

  ZCatalystRow fromJson(Map<String, dynamic> json,
      Function(ZCatalystException exception) onFailed) {
    if (json["ROWID"] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Row ID must not be null'));
    } else if (json["CREATORID"] == null) {
      return onFailed(
          ZCatalystSDKException(message: 'Creator ID must not be null'));
    }
    Map<String, dynamic> dynamicValues = {};
    for (var key in json.keys) {
      if (key != 'ROWID' &&
          key != 'CREATORID' &&
          key != 'CREATEDTIME' &&
          key != 'MODIFIEDTIME') {
        dynamicValues[key] = json[key];
      }
    }
    ZCatalystRow row = ZCatalystRow(tableIdentifier);
    row._columnValues = dynamicValues;
    row.id = json["ROWID"].toString();
    row.creatorId = json["CREATORID"].toString();
    row.createdTime = json["CREATEDTIME"];
    row.modifiedTime = json["MODIFIEDTIME"];
    row._isCreate = false;
    return row;
  }
}
