part of '../api/api_request.dart';

class ZCatalystRow {
  ///The unique **id** of the table in which the current row is present.
  final String _tableIdentifier;

  ///The unique **id** of the row.
  String? id;

  ///The **id** of the user who created this row.
  String? creatorId;

  ///The time in Date-Time format at which the row is created.
  String? createdTime;

  ///The last modified time of the row.
  String? modifiedTime;

  Map<String, dynamic> _columnValues = {};

  final Map<String, dynamic> _upsertMap = {};

  late bool _isCreate;

  ZCatalystRow(this._tableIdentifier);

  ///Returns a [Map] that contains the user-defined columns and its values.
  Map<String, dynamic> getData() {
    var columnVsValueMap = _columnValues;
    columnVsValueMap.addAll(_upsertMap);
    return columnVsValueMap;
  }

  ///Sets a [value] for a column [columnName].
  void setColumnValue(String columnName, dynamic value) {
    _upsertMap[columnName] = value;
  }

  ///Returns the value of a column [columnName].
  dynamic getColumnValue(String columnName) {
    if (_upsertMap.containsKey(columnName)) {
      return _upsertMap[columnName];
    } else {
      return _columnValues[columnName];
    }
  }

  ///Resets the modified value of a column.
  void resetModifiedValue(String columnName) {
    _upsertMap.remove(columnName);
  }

  ///Resets the modified values of all the columns.
  void resetModifiedValues() {
    _upsertMap.clear();
  }

  ///Creates a new row in the current table.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] & the created [ZCatalystRow] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> create(
      {required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    if (_isCreate) {
      await RowAPIHandler(tableIdentifier: _tableIdentifier)
          .createRow(row: this, onSuccess: onSuccess, onFailed: onFailed);
    } else {
      onFailed(ZCatalystException(
          code: ErrorCode.INVALID_DATA, errorMessage: ErrorMessage.INVALID_ID));
    }
  }

  ///Updates an existing row in the current table.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] & the updated [ZCatalystRow] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> update(
      {required Function(APIResponse response, ZCatalystRow row) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    if (!_isCreate) {
      await RowAPIHandler(tableIdentifier: _tableIdentifier)
          .updateRow(row: this, onSuccess: onSuccess, onFailed: onFailed);
    } else {
      onFailed(ZCatalystException(
          code: ErrorCode.INVALID_OPERATION,
          errorMessage: ErrorMessage.CREATE_ONLY_OPERATION));
    }
  }

  ///Deletes an existing row in the current table.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> delete(
      {required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    if (!_isCreate) {
      await RowAPIHandler(tableIdentifier: _tableIdentifier)
          .deleteRow(id: id!, onSuccess: onSuccess, onFailed: onFailed);
    } else {
      onFailed(ZCatalystException(
          code: ErrorCode.INVALID_OPERATION,
          errorMessage: ErrorMessage.CREATE_ONLY_OPERATION));
    }
  }
}
