import 'package:http/http.dart';

import '../api/response/base_api_response.dart';
import '../api/api_request.dart';
import '../exception/zcatalyst_exception.dart';
import '../crud/zcatalyst_user_delegate.dart';

class ZCatalystFile {
  ///The unique **id** of the folder in which the current file is present.
  int folderId;

  ///The unique **id** of the file.
  int id;

  ///The name of the file.
  String name;

  ///The path at which the file is located.
  String? location;

  ///The size of the file in bytes.
  int size;

  ///The time in Date-Time format at which the current file is created.
  String createdTime;

  ///The details of the user who created this file.
  ZCatalystUserDelegate createdBy;

  ///The time in Date-Time format at which the current file is modified.
  String modifiedTime;

  ///The details of the user who recently modified this file.
  ZCatalystUserDelegate modifiedBy;

  ZCatalystFile(
      {required this.folderId,
      required this.id,
      required this.name,
      this.location,
      required this.size,
      required this.createdTime,
      required this.createdBy,
      required this.modifiedTime,
      required this.modifiedBy});

  ///Deletes the current file from the folder.
  ///
  ///On Success, The **onSuccess** handler is called with the [APIResponse] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> delete(
      {required Function(APIResponse response) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await FileAPIHandler(folderId: folderId)
        .deleteFile(id, onSuccess, onFailed);
  }

  ///Downloads the current file from the folder and writes to the [filePath] with the file name as [fileName].
  ///
  ///On Download Progress, The **onProgress** handler is called with the no of [bytesWritten], [contentLength], [percentage] in realtime.
  ///
  ///On Success, The **onSuccess** handler is called with the [FileAPIResponse] object & the [filePath] at which the file is downloaded.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> download(
      {required String filePath,
      required String fileName,
      required Function(FileAPIResponse response, String filePath) onSuccess,
      required Function(int bytesWritten, int contentLength, double percentage)
          onProgress,
      required Function(ZCatalystException exception) onFailed}) async {
    await FileAPIHandler(folderId: folderId).downloadFile(id,
        path: filePath,
        name: fileName,
        onSuccess: onSuccess,
        onProgress: onProgress,
        onFailed: onFailed);
  }

  ///Downloads the current file from the folder and returns a [ByteStream] that you can listen to and download.
  ///
  ///On Success, The **onSuccess** handler is called with the [FileAPIResponse] object & the [ByteStream].
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> downloadAsStream(
      {required Function(FileAPIResponse response, ByteStream fileStream)
          onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await FileAPIHandler(folderId: folderId)
        .downloadFileAsStream(id, onSuccess: onSuccess, onFailed: onFailed);
  }
}
