import '../api/api_request.dart';
import '../api/response/base_api_response.dart';
import '../crud/zcatalyst_user.dart';
import '../crud/zcatalyst_user_delegate.dart';
import '../exception/zcatalyst_exception.dart';
import '../exception/zcatalyst_logger.dart';
import '../filestore/zcatalyst_file_store.dart.dart';
import '../function/zcatalyst_function.dart';
import '../util/api_constants.dart';
import 'package:flutter/services.dart';
import '../exception/zcatalyst_sdk_exception.dart';

part 'zcatalyst_sdk_configs.dart';
part '../auth/iam_client_sdk_plugin.dart';

class ZCatalystApp {
  ///The Catalyst SDK Configuration Object.
  static ZCatalystSDKConfigs configs = ZCatalystSDKConfigs._();

  ZCatalystApp._internal();

  static ZCatalystApp? _sdkInstance;

  ///Initializes the Catalyst SDK and creates a singleton instance of the [ZCatalystApp] class.
  ///
  ///If [sdkConfigs] object is provided, The initialization parameters are taken from this object.
  ///Otherwise, The parameters are taken from the imported catalyst configuration file.
  ///
  ///Throws [ZCatalystException] with the following error codes:
  ///- **INITIALIZATION_ERROR**:
  ///  - Thrown if the conifguration file is not present at the appropriate path.
  ///- **INTERNAL_ERROR**:
  ///  - Thrown if the error is caused by the SDK itself.
  static Future<void> init(
      {bool isCustomLogin = false,
      ZCatalystSDKConfigs? sdkConfigs,
      ZCatalystEnvironment? environment}) async {
    try {
      if (sdkConfigs != null) {
        await _initWithSDKConfigs(sdkConfigs);
      } else {
        await _initWithPropertiesFile(isCustomLogin, environment);
      }
    } on ZCatalystException {
      rethrow;
    }
  }

  static Future<void> _initWithSDKConfigs(
      ZCatalystSDKConfigs sdkConfigs) async {
    _sdkInstance ??= ZCatalystApp._internal();
    configs = sdkConfigs;
    try {
      await IAMClientSDKPlugin._initWithSDKConfigs(sdkConfigs);
      configs._initLogger();
    } on ZCatalystException {
      rethrow;
    }
  }

  static Future<void> _initWithPropertiesFile(
      bool isCustomLogin, ZCatalystEnvironment? environment) async {
    _sdkInstance ??= ZCatalystApp._internal();
    environment = environment ?? ZCatalystEnvironment.PRODUCTION;
    Map<String, dynamic> configsJson;
    try {
      configsJson = await IAMClientSDKPlugin._init(environment, isCustomLogin);
    } on ZCatalystException {
      rethrow;
    }
    configs = ZCatalystSDKConfigs._()
      .._environment = environment
      .._accountsPortalBaseURL =
          configsJson[SDKConfigKeys.ACCOUNTS_PORTAL_DOMAIN]
      .._clientID = configsJson[SDKConfigKeys.CLIENT_ID]
      .._clientSecret = configsJson[SDKConfigKeys.CLIENT_SECRET]
      .._apiBaseURL = configsJson[SDKConfigKeys.API_BASE_URL]
      .._projectID = configsJson[SDKConfigKeys.PROJECT_ID]
      .._portalID = configsJson[SDKConfigKeys.PORTAL_ID]
      .._redirectURLScheme = configsJson[SDKConfigKeys.REDIRECT_URL_SCHEME]
      .._oauthScopes = configsJson[SDKConfigKeys.SCOPES]
      .._printStackTrace = configsJson[SDKConfigKeys.PRINT_STACK_TRACE] ?? false
      .._jwtClientID = configsJson[SDKConfigKeys.JWT_CLIENT_ID]
      .._jwtClientSecret = configsJson[SDKConfigKeys.JWT_CLIENT_SECRET]
      .._isCustomLoginEnabled = isCustomLogin
      .._requestHeaders =
          Map<String, String>.from(configsJson[SDKConfigKeys.REQUEST_HEADERS]);
    configs._initLogger();
  }

  ///Returns a singleton instance of the [ZCatalystApp] class.
  static ZCatalystApp getInstance() {
    if (_sdkInstance == null) {
      _sdkInstance = ZCatalystApp._internal();
      return _sdkInstance!;
    }
    return _sdkInstance!;
  }

  ///Constructs and returns a [ZCatalystUserDelegate] object required for user sign-up.
  ZCatalystUserDelegate newUser(
      {required String lastName, required String emailId}) {
    return ZCatalystUserDelegate(lastName, emailId);
  }

  ///Creates a new catalyst user account using the values from [ZCatalystUserDelegate] object.
  ///
  ///On success, The **onSuccess** handler is called with an [APIResponse] & [ZCatalystUser] object.
  ///
  ///On errors, The **OnFailed** handler is called with a [ZCatalystException].
  Future<void> signUp(
      {required ZCatalystUserDelegate newUser,
      required Function(APIResponse response, ZCatalystUser user) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await UserAPIHandler()
        .signUp(newUser: newUser, onSuccess: onSuccess, onFailed: onFailed);
  }

  ///Displays a login screen to the user.
  ///
  ///On success, The **onSuccess** handler is called.
  ///
  ///On errors, The **OnFailed** handler is called with a [ZCatalystException].
  Future<void> login(
      {Map<String, String>? customParams,
      required Function() onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    if (!configs.isCustomLoginEnabled) {
      await IAMClientSDKPlugin._login(
          customParams: customParams, onSuccess: onSuccess, onFailed: onFailed);
    } else {
      return onFailed(ZCatalystException(
          code: ErrorCode.INVALID_CONFIGURATION,
          errorMessage: ErrorMessage.INVALID_CONFIGURATION_FOR_DEFAULT_LOGIN));
    }
  }

  ///Completes the Third Party Authentication Flow and logs in the user.
  ///
  ///On success, The **onSuccess** handler is called.
  ///
  ///On errors, The **OnFailed** handler is called with a [ZCatalystException].
  Future<void> handleCustomLogin(
      {required String jwtToken,
      required Function() onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    try {
      if (configs.isCustomLoginEnabled) {
        await IAMClientSDKPlugin._getOauthToken(jwtToken: jwtToken);
        onSuccess();
      } else {
        onFailed(ZCatalystException(
            code: ErrorCode.INVALID_CONFIGURATION,
            errorMessage: ErrorMessage.INVALID_CONFIGURATION_FOR_CUSTOM_LOGIN));
      }
    } on ZCatalystException catch (ex) {
      onFailed(ex);
    }
  }

  ///Whether the current user is logged in.
  ///
  ///Returns [True] if the user is logged in. Otherwise, returns [False].
  Future<bool> isUserLoggedIn() async =>
      await IAMClientSDKPlugin._isUserLoggedIn();

  ///Returns the access token of the current user if logged in.
  Future<String> getAccessToken() async {
    try {
      return await IAMClientSDKPlugin._getAccessToken();
    } on ZCatalystException {
      rethrow;
    }
  }

  ///Fetches the details of the current logged in user.
  ///
  ///On success, The **onSuccess** handler is called with the [APIResponse] & [ZCatalystUser] object.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> getCurrentUser(
      {required Function(APIResponse response, ZCatalystUser user) onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await UserAPIHandler()
        .getCurrentUser(onSuccess: onSuccess, onFailed: onFailed);
  }

  ///Returns a new instance of the [ZCatalystDataStore] class.
  ZCatalystDataStore getDataStoreInstance() {
    return ZCatalystDataStore();
  }

  ///Returns a new instance of the [ZCatalystFileStore] class.
  ZCatalystFileStore getFileStoreInstance() {
    return ZCatalystFileStore();
  }

  ///Returns a new instance of the [ZCatalystFunction] class.
  ZCatalystFunction getFunctionInstance({required String identifier}) {
    return ZCatalystFunction(identifier);
  }

  ///Performs a Catalyst **Search** Operation with the given [ZCatalystSearchOptions] object.
  ///
  ///On success, The **onSuccess** handler is called with the [APIResponse] object and the **search result**.
  ///
  ///On errors, The **onFailed** handler is called with a [ZCatalystException].
  Future<void> search(
      ZCatalystSearchOptions searchOptions,
      Function(APIResponse response, Map<String, dynamic> result) onSuccess,
      Function(ZCatalystException exception) onFailed) async {
    await SearchAPIhandler(searchOptions).search(onSuccess, onFailed);
  }

  ///Signs out the current user from the app.
  ///This also means that the access token of the current user will be revoked.
  Future<void> logout(
      {required Function() onSuccess,
      required Function(ZCatalystException exception) onFailed}) async {
    await IAMClientSDKPlugin._logout(onSuccess: onSuccess, onFailed: onFailed);
  }
}
